#!/usr/bin/env node

import { OptionValues, program } from "commander"
import inquirer from "inquirer"
import { loadYaml } from "../../utils/load-yaml"
import shell from "shelljs"
import { getLogger } from "../../utils/logging"
import { existsSync, mkdirSync, readdir, readFileSync, writeFile, writeFileSync } from "fs"
import { watch } from "chokidar"
import { resolve, basename, dirname } from "path"
import { redisHandler } from "../../utils/redis"
import { doQuestion, Question } from "./question"
import { publishOutfile } from "../commons/publishOutfile"
export const logger = getLogger()
const cfgobj = loadYaml("config/main.cfg.yaml")

program
  .option("-p, --pro <string>", "项目名称")
  .option("-m, --model <string>", "模块名称")
  .option("-s, --subModel <string>", "子模块名称")
  .option("-f, --fileName <string>", "文件名")
program.parse(process.argv)
const opts = program.opts()

interface FileName {
  fileName: string
}

async function mainProcess(opts: OptionValues) {

  let question: Question;
  const { pro, model, subModel } = opts
  if (pro && model && subModel) {
    question = {
      projectName: pro,
      modelName: model,
      subModelName: subModel
    }
  } else {
    // 询问项目名/模块名和子模块名
    const q = await doQuestion();
    question = q!
  }
  if (question == null) {
    logger.error("question is not complate...!")
    return;
  }

  // 询问文件名
  let fName: FileName
  if (opts.fileName) {
    fName = {
      fileName: opts.fileName
    }
  } else {
    fName = await inquirer.prompt({
      name: "fileName",
      type: "input",
      message: "文件名"
    })
  }

  // 创建文件
  const fileName = resolve(cfgobj.question.cached_dir, question.projectName, question.modelName, question.subModelName, fName.fileName + ".yaml")
  const cachedName = resolve(cfgobj.question.struct_cached_dir, question.projectName, question.modelName, question.subModelName, fName.fileName + ".json")
  if (!existsSync(fileName)) {
    const theDir = dirname(fileName)
    // 如果目录不存在需要先创建目录
    if (!existsSync(theDir)) {
      mkdirSync(theDir, { recursive: true }) // 递归创建目录
    }
    writeFile(fileName, "", err => {
      if (err) {
        logger.error(err)
      } else {
        logger.info("创建文件成功...!")
      }
    })
  } // end if

  // 将本地配置缓存推送到配置端
  const cachedPth = resolve("node_modules", question.projectName, "caches", question.modelName, question.subModelName, cfgobj.model.cache_file_name)
  const metasPth = resolve("node_modules", question.projectName, "metas", question.modelName, question.subModelName, cfgobj.model.metas_file_name)
  const metascriptPth = resolve("node_modules", question.projectName, "src", question.modelName, question.subModelName, cfgobj.model.metascript_file_name)
  const targetCachedPth = resolve(cfgobj.question.struct_cached_dir, question.projectName, question.modelName, question.subModelName, fName.fileName + ".json")
  if (existsSync(metascriptPth)) {
    const { getMetaInfo } = require(metascriptPth)
    const { title, treeCode, treeData } = getMetaInfo(question.projectName, question.modelName, question.subModelName, fName.fileName)
    logger.info(`title: ${title}; `)
    if (treeData) {
      pushCachedStruct(title, JSON.stringify({ code: treeCode, data: treeData }))
    } else {
      pushCached(title, treeCode)
    }
    logger.info("emit-script meta success")
  } else if (existsSync(targetCachedPth)) { // 存在目标文件的缓存
    logger.info("use file cached...")
    pushCachedStruct(question.modelName + "-" + question.subModelName + ":" + fName.fileName
      , readFileSync(targetCachedPth, { encoding: "utf-8" }))
    logger.info("emit-code success")
  } else if (existsSync(cachedPth)) { // 存在针对模块的缓存文件
    logger.info("use model cached....")
    redisHandler(clt => {
      // 设置配置文件内容到redis
      clt.hset("config-cached-struct", "title", question.modelName + "-" + question.subModelName + ":" + fName.fileName
        , "treeCode", readFileSync(cachedPth, { encoding: "utf-8" }), num => {
          logger.info(`publish ${cfgobj.redis.channel} code-struct-cached...`)
          clt.publish(cfgobj.redis.channel, "code-struct-cached") // 推送代码结构
          clt.quit()
        })

    })
    logger.info("emit-code success")
  } else if (existsSync(metasPth)) { // 否则使用元配置文件
    logger.info("use meta...")
    redisHandler(clt => {
      const titlePth = resolve("node_modules", question.projectName, "metas", question.modelName, question.subModelName, cfgobj.model.title_file_name)

      // 设置配置文件内容到redis
      clt.hset("config-struct", "title", readFileSync(titlePth, { encoding: "utf-8" })
        , "treeCode", readFileSync(metasPth, { encoding: "utf-8" }), num => {
          logger.info(`publish ${cfgobj.redis.channel} code-struct...`)
          clt.publish(cfgobj.redis.channel, "code-struct") // 推送代码结构
          clt.quit()
        })

    })
    logger.info("emit-code success")
  } else {
    throw new TypeError("没有对应的缓存文件")
  }

  // 关联配置端到输出文件
  publishOutfile(fileName, cachedName)


  const outfilesCached = {
    outfile: fileName,
    cacheFile: cachedName
  }

  writeFile('.cached_outfiles', JSON.stringify(outfilesCached), (err) => {
    if (err) {
      logger.error("cached_outfiles write fail...")
    } else {
      logger.info("cached_outfiles write success...")
    }
  })

}
mainProcess(opts);



function pushCachedStruct(title: any, treeCode: any) {
  redisHandler(clt => {
    // 设置配置文件内容到redis
    clt.hset("config-cached-struct", "title", title,
      "treeCode", treeCode, num => {
        logger.info(`publish ${cfgobj.redis.channel} code-struct-cached...`)
        clt.publish(cfgobj.redis.channel, "code-struct-cached") // 推送代码结构
        clt.quit()
      })

  })
}

function pushCached(title: any, treeCode: any) {
  redisHandler(clt => {
    // 设置配置文件内容到redis
    clt.hset("config-struct", "title", title,
      "treeCode", treeCode, num => {
        logger.info(`publish ${cfgobj.redis.channel} code-struct...`)
        clt.publish(cfgobj.redis.channel, "code-struct") // 推送代码结构
        clt.quit()
      })
  })
}

