
import { loadYaml } from "../../utils/load-yaml"
import inquirer from "inquirer"
import { getLogger } from "../../utils/logging"
import { existsSync, readdirSync } from "fs"
import { resolve } from "path"


const logger = getLogger();
const questionPath = 'config/watch/question.yaml'
const prefixModel = "mill-template-"
const questionObj: inquirer.QuestionCollection[] = loadYaml(questionPath)
const [project, model, subModel] = questionObj

interface Question {
    projectName: string;
    modelName: string;
    subModelName: string;
}

class QuestionError extends TypeError {

}



/**
 * 交互询问项目名称
 * @param answer 问题
 * @returns 项目名称
 */
async function getProjectName(answer: inquirer.QuestionCollection<inquirer.Answers>) {
    const basePath = resolve("node_modules")
    if (!existsSync(basePath)) {
        throw new QuestionError("the dir node_modules is not exists...")
    }

    const files = readdirSync(basePath).filter(el => el.startsWith(prefixModel))
    if (!files.length) {
        throw new QuestionError("not installed mill-templates, plase install its...")
    }

    (answer as any).choices = files.map(el => {
        return {
            name: el,
            value: el
        }
    })

    const projectName = await inquirer.prompt(answer)
    // TODO choices 需要从客户目录中找mill-templates
    // const choices = (answer as any).choices
    // console.log(choices)
    return projectName.project;
}

/**
 * 交互询问模块名称
 * @param projectName 项目名称
 * @param answer 问题
 * @returns
 */
async function getModelName(projectName: string, answer: inquirer.QuestionCollection<inquirer.Answers>) {
    const basePath = resolve("node_modules", projectName, "src")
    if (!existsSync(basePath)) {
        throw new QuestionError("the module has not src dir...")
    }
    const files = readdirSync(basePath);

    (answer as any).choices = files.map(el => {
        return {
            name: el,
            value: el
        }
    })
    const modelName = await inquirer.prompt(answer)
    // TODO 从指定的projectName找到model，供用户选择
    // throw new QuestionError("question ...");
    return modelName.model;
}

/**
 *
 * @param projectName 项目名称
 * @param modelName 模块名称
 * @param answer
 * @returns
 */
async function getSubModelName(projectName: string, modelName: string, answer: inquirer.QuestionCollection<inquirer.Answers>) {
    // TODO 从子模块名找到脚本供用户选择
    const basePath = resolve("node_modules", projectName, "src", modelName)
    if (!existsSync(basePath)) {
        throw new QuestionError("the module has not sub module...")
    }
    const files = readdirSync(basePath);
    (answer as any).choices = files.map(el => {
        return {
            name: el,
            value: el
        }
    })
    const subModelName = await inquirer.prompt(answer)
    return subModelName.sub_model;
}

async function doQuestion(): Promise<Question | null> {
    try {
        const projectName = await getProjectName(project)
        const modelName = await getModelName(projectName, model)
        const subModelName = await getSubModelName(projectName, modelName, subModel)
        const question = {
            projectName, modelName, subModelName
        }
        // console.log(question)
        return question;
    } catch (error) {
        logger.error(error.message)
    }
    return null;
}

export {
    doQuestion, Question
}